from image_fetcher.tools import escape_image_name


class TestTools:
    def test_escape_image_name_http(self):
        url = "http://www.fakewebsite.com/sample.jpg"
        result = escape_image_name(url)
        assert result == "wwwfakewebsitecomsample.jpg"

    def test_escape_image_name_https(self):
        url = "https://www.fakewebsite.com/sample.jpg"
        result = escape_image_name(url)
        assert result == "wwwfakewebsitecomsample.jpg"

    def test_escape_image_name_non_alphanumeric_characters(self):
        url = "https://www.fakewebsite!.com/sample$_.jpg"
        result = escape_image_name(url)
        assert result == "wwwfakewebsitecomsample.jpg"



import os
from concurrent.futures import ThreadPoolExecutor, wait, as_completed
from func_timeout import func_timeout, FunctionTimedOut
from tqdm import tqdm

from image_fetcher.tools import escape_image_name, download_url


def download_image_with_timeout(url: str, timeout: int, output_directory: str):
    """
    Downloads image from given URL. Will exit if not complete within timeout seconds. Doesn't validate input params.

    Parameters:
    url (str): URL to try and download image from
    timeout (int): seconds to wait for function to execute
    output_directory (str): output_directory to save image to

    Returns:
    bool: whether image was downloaded before timeout
    """
    try:
        func_timeout(
            timeout,
            download_image,
            args=(
                url,
                output_directory,
            ),
        )
        return True
    except FunctionTimedOut:
        return False


def download_image(url: str, output_directory: str):
    """
    Downloads image from given URL.

    Parameters:
    url (str): URL to try and download image from
    output_directory (str): output_directory to save image to
    """
    image_name = escape_image_name(url)
    data = download_url(url)
    with open(os.path.join(output_directory, image_name), "wb") as output_file:
        output_file.write(data)


def multi_thread_image_download(
    urls: str,
    output_directory: str,
    max_fetching_threads=None,
    download_timeout=5,
    verbose=True,
):
    """
    Downloads list of images using multiple threads.

    Parameters:
    url_file_path (list/str): list of URLs or path to text file containing list of URLs
    output_directory (str): destination directory path
    max_image_fetching_threads (int): maximum number of concurrent image download threads (default is cores * 5)
    image_download_timeout (int): maximum wait time in seconds for an image download (default is 5)
    verbose (bool): show tqdm progress bar (default true)

    Returns:
    int: total files in the directory
    """
    # If urls is not a list is must be a path to a file with a list of urls
    if isinstance(urls, str):
        with open(urls, "r") as url_file:
            urls = set(url_file.read().splitlines())

    if not os.path.isdir(output_directory):
        os.mkdir(output_directory)
    else:
        # Exclude existing images
        urls = [url for url in urls if escape_image_name(url) not in os.listdir(output_directory)]

    # Build concurrent thread pool with max_image_fetching_threads
    with ThreadPoolExecutor(max_fetching_threads) as pool:
        futures = [
            pool.submit(
                download_image_with_timeout,
                url,
                download_timeout,
                output_directory,
            )
            for url in urls
        ]
        if verbose:
            for _ in tqdm(as_completed(futures)):
                pass
        else:
            wait(futures)

    return len(os.listdir(output_directory))



import os
from shutil import rmtree

from image_fetcher.tools import escape_image_name
from image_fetcher.download_images import download_image, download_image_with_timeout, multi_thread_image_download


def reset_directory(directory):
    if os.path.isdir(directory):
        rmtree(directory)
    os.mkdir(directory)


IMAGE_DIRECTORY = "images"
URL_FILE = os.path.join("tests", "urls.txt")
with open(URL_FILE) as f:
    URLS = [line.strip() for line in f.readlines()]



class TestDownloadImages:
    def test_download_image(self):
        reset_directory(IMAGE_DIRECTORY)
        url = URLS[0]

        download_image(url, IMAGE_DIRECTORY)

        file_name = escape_image_name(url)
        assert file_name in os.listdir(IMAGE_DIRECTORY)
        assert os.path.getsize(os.path.join(IMAGE_DIRECTORY, file_name)) > 0

    def test_download_image_with_timeout_within_time(self):
        reset_directory(IMAGE_DIRECTORY)
        timeout = 5
        url = URLS[0]

        successful = download_image_with_timeout(url, timeout, IMAGE_DIRECTORY)

        assert successful
        file_name = escape_image_name(url)
        assert file_name in os.listdir(IMAGE_DIRECTORY)
        assert os.path.getsize(os.path.join(IMAGE_DIRECTORY, file_name)) > 0

    def test_download_image_with_timeout_exceed_time(self):
        reset_directory(IMAGE_DIRECTORY)
        timeout = 0
        url = URLS[0]

        successful = download_image_with_timeout(url, timeout, IMAGE_DIRECTORY)

        assert not successful
        file_name = escape_image_name(url)
        assert file_name not in os.listdir(IMAGE_DIRECTORY)

    def test_multi_thread_image_download(self):
        reset_directory(IMAGE_DIRECTORY)

        total_images = multi_thread_image_download(
            URL_FILE,
            IMAGE_DIRECTORY,
            max_fetching_threads=2,
            download_timeout=5,
            verbose=False,
        )
        assert total_images == len(URLS)
        for url in URLS:
            file_name = escape_image_name(url)
            assert file_name in os.listdir(IMAGE_DIRECTORY)
            assert os.path.getsize(os.path.join(IMAGE_DIRECTORY, file_name)) > 0

    def test_multi_thread_image_download_list(self):
        reset_directory(IMAGE_DIRECTORY)

        total_images = multi_thread_image_download(
            [
                "https://benaandrew.github.io/images/sentiment.jpg",
                "https://benaandrew.github.io/images/dog.jpg"
            ],
            IMAGE_DIRECTORY,
            max_fetching_threads=2,
            download_timeout=5,
            verbose=False,
        )
        assert total_images == len(URLS)
        for url in URLS:
            file_name = escape_image_name(url)
            assert file_name in os.listdir(IMAGE_DIRECTORY)
            assert os.path.getsize(os.path.join(IMAGE_DIRECTORY, file_name)) > 0

    @classmethod
    def teardown_class(cls):
        rmtree(IMAGE_DIRECTORY)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="image_fetcher",
    version="2.0.0",
    author="Ben Andrew",
    author_email="benandrew89@gmail.com",
    description="A simple lightweight library to download images (and other files)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/BenAAndrew/Python-Image-Fetcher",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "func-timeout==4.3.5",
        "tqdm==4.62.0",
        "urllib3==1.26.6",
    ],
    python_requires=">=3.5",
)



from re import sub
from urllib import request


def download_url(url: str):
    """
    Read data from a given URL

    Parameters:
    url (str): URL to retrieve data from

    Returns:
    bytes: Returns bytes of data from the URL
    """
    req = request.Request(url)
    with request.urlopen(req) as resp:
        data = resp.read()
    return data


def escape_image_name(url: str):
    """
    Remove 'http(s)://' and all non alphanumeric characters from file URL

    Parameters:
    url (str): URL to escape

    Returns:
    str: Escaped URL
    """
    extension = url.split(".")[-1]
    # Remove extension
    escaped = url[: -len(extension) - 1]
    # Remove http:// or https://
    escaped = escaped.split("/", 1)[1]
    # Remove all non alphanumeric characters
    return f"{sub('[^a-zA-Z0-9]+', '', escaped)}.{extension}"
